#ifndef __LC3_H__
#define __LC3_H__

/*
 * "Copyright (c) 2012 by Fritz Sieker."
 * based on ideas/code by Steven S. Lumetta
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written
 * agreement is hereby granted, provided that the above copyright notice
 * and the following two paragraphs appear in all copies of this software,
 * that the files COPYING and NO_WARRANTY are included verbatim with
 * any distribution, and that the contents of the file README are included
 * verbatim as part of a file named README with any distribution.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE AUTHOR
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 */

/** @file lc3.h
 *  @brief definitions of the LC3 instruction set architecture (ISA) (do not modify)
 *  @details This defines the details of the LC3 instruction set architecture
 *  (ISA). It is a separate file so that it could be shared by both an assembler
 *  (not yet completed) and the simulator.
 * <p>
 * @author Fritz Sieker
 */

/** LC3 words are 16 bits */
#define LC3_WORD unsigned short

/** The LC3 defines a memory accessed by a 16 bit address */
#define LC3_MEM_SIZE 65538

/** The LC3 contains 8 general purpose register, named R0..R7 */
#define LC3_NUM_REGS 8  

/** Return address stored in R7 */
#define RETURN_ADDR_REG   7

/** The sixteen LC3 opcodes */
typedef enum opcode_t {
  OP_BR = 0,
  OP_ADD,
  OP_LD,
  OP_ST,
  OP_JSR,
  OP_AND,
  OP_LDR,
  OP_STR,
  OP_RTI,
  OP_NOT,
  OP_LDI,
  OP_STI,
  OP_JMP_RET,

#if defined SHIFT_OP
  OP_SHIFT,
#elif defined STACK_OP
  OP_PUSHPOP,
#elif defined FLADD_OP
  OP_FLADD,
#else
  OP_RESERVED,
#endif

  OP_LEA,
  OP_TRAP,
  NUM_OPS
} opcode_t;

/** A bit field used to define the  types of parameters an individual
 *  LC3 instruction may have. Each value represents a different bit in
 *  the final result. When you see C code like this, it is likely that
 *  an integer value is used to represent an "array" of up to 32 boolean
 *  values. Each value is accessed with a mask that extracts the bit of
 *  interest. See the inst_format_t below.
 */
typedef enum inst_field {
    FMT_R1    = 0x001, /**< DR or SR                      */
    FMT_R2    = 0x002, /**< SR1 or BaseR                  */
    FMT_R3    = 0x004, /**< SR2                           */
    FMT_CC    = 0x008, /**< condition codes               */
    FMT_IMM5  = 0x010, /**< imm5                          */
    FMT_IMM6  = 0x020, /**< offset6                       */
    FMT_VEC8  = 0x040, /**< trapvect8                     */
    FMT_ASC8  = 0x080, /**< 8-bit ASCII                   */
    FMT_IMM9  = 0x100, /**< label (or address from imm9)  */
    FMT_IMM11 = 0x200, /**< label (or address from imm11) */
    FMT_IMM16 = 0x400  /**< full instruction in hex       */
} inst_field_t;

/** Define a combinatin of parameters an opcode may have. For example, the
 *  the BR, LD, LDI, ST and STI instructions all have two parameters. The
 *  first is a register, the second is a nine bit offset. This form stores
 *  multiple boolean values in a single integer values by using individual
 *  bits to encode information.
 */
typedef enum inst_format {
  FMT_      = 0,
  FMT_RRR   = (FMT_R1 | FMT_R2 | FMT_R3),
  FMT_RRI5  = (FMT_R1 | FMT_R2 | FMT_IMM5),
  FMT_CL    = (FMT_CC | FMT_IMM9),
  FMT_R     = FMT_R2,
  FMT_I11   = FMT_IMM11,
  FMT_RL    = (FMT_R1 | FMT_IMM9),
  FMT_RRI6  = (FMT_R1 | FMT_R2 | FMT_IMM6),
  FMT_RR    = (FMT_R1 | FMT_R2),
  FMT_V     = FMT_VEC8,
  FMT_A     = FMT_ASC8,
  FMT_16    = FMT_IMM16
} inst_format_t;

/** This structure stores the information about a single instruction.
 *  See the usage in lc3.c where the information for each LC3 instruction
 *  is defined.
 */
typedef struct inst_info {
  char*        name[2];    /**< printable name(s) of instruction e.g. "ADD" */
  opcode_t     opcode;     /**< opcode of this instruction */
  int          opBit;      /**< -1 if instruction has only one format,
                                else bit that differentiates format.
                                (e.g. bit 5 for ADD/AND) */
  inst_field_t format[2];  /**< primary/secondary parameter format */
} inst_info_t;

/** Get the information for an instruction, given its opcode This is simply
 *  an access into an array of values initialized with the information for
 *  each of the LC3's sixteen instructions.
 */
inst_info_t* lc3_get_inst_info (opcode_t opcode);

#endif /* __LC3_H__ */

